/*
  pins_arduino.c - pin definitions for the Arduino board
  Part of Arduino / Wiring Lite

  Copyright (c) 2005 David A. Mellis
  Modified 31 August 2010 by Alex Galkin

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA

  $Id: pins_arduino.c 565 2009-03-25 10:50:00Z dmellis $
*/

#include <avr/io.h>
#include "wiring_private.h"
#include "pins_arduino.h"

// On the Arduino board, digital pins are also used
// for the analog output (software PWM).  Analog input
// pins are a separate set.

// ATMEL ATMEGA8 & 168 / ARDUINO
//
//                  +-\/-+
//            PC6  1|    |28  PC5 (AI 5)
//      (D 0) PD0  2|    |27  PC4 (AI 4)
//      (D 1) PD1  3|    |26  PC3 (AI 3)
//      (D 2) PD2  4|    |25  PC2 (AI 2)
// PWM+ (D 3) PD3  5|    |24  PC1 (AI 1)
//      (D 4) PD4  6|    |23  PC0 (AI 0)
//            VCC  7|    |22  GND
//            GND  8|    |21  AREF
//            PB6  9|    |20  AVCC
//            PB7 10|    |19  PB5 (D 13)
// PWM+ (D 5) PD5 11|    |18  PB4 (D 12)
// PWM+ (D 6) PD6 12|    |17  PB3 (D 11) PWM
//      (D 7) PD7 13|    |16  PB2 (D 10) PWM
//      (D 8) PB0 14|    |15  PB1 (D 9) PWM
//                  +----+
//
// (PWM+ indicates the additional PWM pins on the ATmega168.)

// ATMEL ATMEGA1280 / ARDUINO      // ATMEL ATMEGA128 / ARDUINO   
//                                 //                             
// 0-7 PE0-PE7   works             // 0-7 PD0-PD7   works         
// 8-13 PB0-PB5  works             // 8-15 PC0-PC7  works         
// 14-21 PA0-PA7 works             // 16-23 PA0-PA7 works         
// 22-29 PH0-PH7 works             // 24-31 PB0-PB7 works         
// 30-35 PG5-PG0 works             // 32-39 PE0-PE7 works         
// 36-43 PC7-PC0 works             // 40-47 PF0-PF7 works         
// 44-51 PJ7-PJ0 works             // 48-50 PG0-PG2 works         
// 52-59 PL7-PL0 works                      
// 60-67 PD7-PD0 works                      
// A0-A7 PF0-PF7                            
// A8-A15 PK0-PK7                           

#define PA 1
#define PB 2
#define PC 3
#define PD 4
#define PE 5
#define PF 6
#define PG 7

#define REPEAT8(x) x, x, x, x, x, x, x, x
#define BV0TO7 _BV(0), _BV(1), _BV(2), _BV(3), _BV(4), _BV(5), _BV(6), _BV(7)
#define BV7TO0 _BV(7), _BV(6), _BV(5), _BV(4), _BV(3), _BV(2), _BV(1), _BV(0)

#if defined(__AVR_ATmega128__)
const uint16_t PROGMEM port_to_mode_PGM[] = {
	NOT_A_PORT,
	&DDRA,
	&DDRB,
	&DDRC,
	&DDRD,
	&DDRE,
	&DDRF,
	&DDRG,
};

const uint16_t PROGMEM port_to_output_PGM[] = {
	NOT_A_PORT,
	&PORTA,
	&PORTB,
	&PORTC,
	&PORTD,
	&PORTE,
	&PORTF,
	&PORTG,
};

const uint16_t PROGMEM port_to_input_PGM[] = {
	NOT_A_PIN,
	&PINA,
	&PINB,
	&PINC,
	&PIND,
	&PINE,
	&PINF,
	&PING,
};

const uint8_t PROGMEM digital_pin_to_port_PGM[] = {
	// PORTLIST		
	// -------------------------------------------		
	PD	, // PD 0 ** 0 ** I2C_SCL INT0
	PD	, // PD 1 ** 1 ** I2C_SDA INT1
	PD	, // PD 2 ** 2 ** USART1_RX INT2
	PD	, // PD 3 ** 3 ** USART1_TX INT3
	PD	, // PD 4 ** 4 ** D4	
	PD	, // PD 5 ** 5 ** D5	
	PD	, // PD 6 ** 6 ** D6	
	PD	, // PD 7 ** 7 ** D7	
	PC	, // PC 0 ** 8 ** D8	
	PC	, // PC 1 ** 9 ** D9	
	PC	, // PC 2 ** 10 ** D10	
	PC	, // PC 3 ** 11 ** D11	
	PC	, // PC 4 ** 12 ** D12	
	PC	, // PC 5 ** 13 ** D13	
	PC	, // PC 6 ** 14 ** D14	
	PC	, // PC 7 ** 15 ** D15	
	PA	, // PA 0 ** 16 ** D16	
	PA	, // PA 1 ** 17 ** D17	
	PA	, // PA 2 ** 18 ** D18	
	PA	, // PA 3 ** 19 ** D19	
	PA	, // PA 4 ** 20 ** D20	
	PA	, // PA 5 ** 21 ** D21	
	PA	, // PA 6 ** 22 ** D22	
	PA	, // PA 7 ** 23 ** D23	
	PB	, // PB 0 ** 24 ** SPI_SS
	PB	, // PB 1 ** 25 ** SPI_SCK
	PB	, // PB 2 ** 26 ** SPI_MOSI
	PB	, // PB 3 ** 27 ** SPI_MISO
	PB	, // PB 4 ** 28 ** PWM	OC0
	PB	, // PB 5 ** 29 ** PWM	OC1A
	PB	, // PB 6 ** 30 ** PWM	OC1B
	PB	, // PB 7 ** 31 ** PWM	OC2/OC1C
	PE	, // PE 0 ** 32 ** USART0_RX
	PE	, // PE 1 ** 33 ** USART0_TX
	PE	, // PE 2 ** 34 ** D34	
	PE	, // PE 3 ** 35 ** PWM OC3A
	PE	, // PE 4 ** 36 ** PWM OC3B INT4
	PE	, // PE 5 ** 37 ** PWM OC3C INT5
	PE	, // PE 6 ** 38 ** INT6
	PE	, // PE 7 ** 39 ** INT7
	PF	, // PF 0 ** 40 ** D40 A0
	PF	, // PF 1 ** 41 ** D41 A1
	PF	, // PF 2 ** 42 ** D42 A2
	PF	, // PF 3 ** 43 ** D43 A3
	PF	, // PF 4 ** 44 ** D44 A4
	PF	, // PF 5 ** 45 ** D45 A5
	PF	, // PF 6 ** 46 ** D46 A6
	PF	, // PF 7 ** 47 ** D47 A7
	PG	, // PG 0 ** 48 ** D48	
	PG	, // PG 1 ** 49 ** D49	
	PG	, // PG 2 ** 50 ** D50	
};

const uint8_t PROGMEM digital_pin_to_bit_mask_PGM[] = {
	// PIN IN PORT		
	// -------------------------------------------		
	_BV( 0 )	, // PD 0 ** 0 ** I2C_SCL INT0      
	_BV( 1 )	, // PD 1 ** 1 ** I2C_SDA INT1             
	_BV( 2 )	, // PD 2 ** 2 ** USART1_RX INT2           
	_BV( 3 )	, // PD 3 ** 3 ** USART1_TX INT3           
	_BV( 4 )	, // PD 4 ** 4 ** D4	                  
	_BV( 5 )	, // PD 5 ** 5 ** D5	                  
	_BV( 6 )	, // PD 6 ** 6 ** D6	                  
	_BV( 7 )	, // PD 7 ** 7 ** D7	                  
	_BV( 0 )	, // PC 0 ** 8 ** D8	                  
	_BV( 1 )	, // PC 1 ** 9 ** D9	                  
	_BV( 2 )	, // PC 2 ** 10 ** D10	                  
	_BV( 3 )	, // PC 3 ** 11 ** D11	                  
	_BV( 4 )	, // PC 4 ** 12 ** D12	                  
	_BV( 5 )	, // PC 5 ** 13 ** D13	                  
	_BV( 6 )	, // PC 6 ** 14 ** D14	                  
	_BV( 7 )	, // PC 7 ** 15 ** D15	                  
	_BV( 0 )	, // PA 0 ** 16 ** D16	                  
	_BV( 1 )	, // PA 1 ** 17 ** D17	                  
	_BV( 2 )	, // PA 2 ** 18 ** D18	                  
	_BV( 3 )	, // PA 3 ** 19 ** D19	                  
	_BV( 4 )	, // PA 4 ** 20 ** D20	                  
	_BV( 5 )	, // PA 5 ** 21 ** D21	                  
	_BV( 6 )	, // PA 6 ** 22 ** D22	                  
	_BV( 7 )	, // PA 7 ** 23 ** D23	                  
	_BV( 0 )	, // PB 0 ** 24 ** SPI_SS                  
	_BV( 1 )	, // PB 1 ** 25 ** SPI_SCK                 
	_BV( 2 )	, // PB 2 ** 26 ** SPI_MOSI                
	_BV( 3 )	, // PB 3 ** 27 ** SPI_MISO                
	_BV( 4 )	, // PB 4 ** 28 ** PWM	OC0               
	_BV( 5 )	, // PB 5 ** 29 ** PWM	OC1A              
	_BV( 6 )	, // PB 6 ** 30 ** PWM	OC1B              
	_BV( 7 )	, // PB 7 ** 31 ** PWM	OC2/OC1C          
	_BV( 0 )	, // PE 0 ** 32 ** USART0_RX               
	_BV( 1 )	, // PE 1 ** 33 ** USART0_TX               
	_BV( 2 )	, // PE 2 ** 34 ** D34	                  
	_BV( 3 )	, // PE 3 ** 35 ** PWM OC3A                
	_BV( 4 )	, // PE 4 ** 36 ** PWM OC3B INT4           
	_BV( 5 )	, // PE 5 ** 37 ** PWM OC3C INT5           
	_BV( 6 )	, // PE 6 ** 38 ** INT6                    
	_BV( 7 )	, // PE 7 ** 39 ** INT7                    
	_BV( 0 )	, // PF 0 ** 40 ** D40 A0                  
	_BV( 1 )	, // PF 1 ** 41 ** D41 A1                  
	_BV( 2 )	, // PF 2 ** 42 ** D42 A2                  
	_BV( 3 )	, // PF 3 ** 43 ** D43 A3                  
	_BV( 4 )	, // PF 4 ** 44 ** D44 A4                  
	_BV( 5 )	, // PF 5 ** 45 ** D45 A5                  
	_BV( 6 )	, // PF 6 ** 46 ** D46 A6                  
	_BV( 7 )	, // PF 7 ** 47 ** D47 A7                  
	_BV( 0 )	, // PG 0 ** 48 ** D48	                  
	_BV( 1 )	, // PG 1 ** 49 ** D49	                  
	_BV( 2 )	, // PG 2 ** 50 ** D50	                  
};

const uint8_t PROGMEM digital_pin_to_timer_PGM[] = {
	// TIMERS		
	// -------------------------------------------		
	NOT_ON_TIMER	, // PD 0 ** 0 ** I2C_SCL INT0        
	NOT_ON_TIMER	, // PD 1 ** 1 ** I2C_SDA INT1        
	NOT_ON_TIMER	, // PD 2 ** 2 ** USART1_RX INT2      
	NOT_ON_TIMER	, // PD 3 ** 3 ** USART1_TX INT3      
	NOT_ON_TIMER	, // PD 4 ** 4 ** D4	              
	NOT_ON_TIMER	, // PD 5 ** 5 ** D5	              
	NOT_ON_TIMER	, // PD 6 ** 6 ** D6	              
	NOT_ON_TIMER	, // PD 7 ** 7 ** D7	              
	NOT_ON_TIMER	, // PC 0 ** 8 ** D8	              
	NOT_ON_TIMER	, // PC 1 ** 9 ** D9	              
	NOT_ON_TIMER	, // PC 2 ** 10 ** D10	              
	NOT_ON_TIMER	, // PC 3 ** 11 ** D11	              
	NOT_ON_TIMER	, // PC 4 ** 12 ** D12	              
	NOT_ON_TIMER	, // PC 5 ** 13 ** D13	              
	NOT_ON_TIMER	, // PC 6 ** 14 ** D14	              
	NOT_ON_TIMER	, // PC 7 ** 15 ** D15	              
	NOT_ON_TIMER	, // PA 0 ** 16 ** D16	              
	NOT_ON_TIMER	, // PA 1 ** 17 ** D17	              
	NOT_ON_TIMER	, // PA 2 ** 18 ** D18	              
	NOT_ON_TIMER	, // PA 3 ** 19 ** D19	              
	NOT_ON_TIMER	, // PA 4 ** 20 ** D20	              
	NOT_ON_TIMER	, // PA 5 ** 21 ** D21	              
	NOT_ON_TIMER	, // PA 6 ** 22 ** D22	              
	NOT_ON_TIMER	, // PA 7 ** 23 ** D23	              
	NOT_ON_TIMER	, // PB 0 ** 24 ** SPI_SS             
	NOT_ON_TIMER	, // PB 1 ** 25 ** SPI_SCK            
	NOT_ON_TIMER	, // PB 2 ** 26 ** SPI_MOSI           
	NOT_ON_TIMER	, // PB 3 ** 27 ** SPI_MISO           
	NOT_ON_TIMER	, // PB 4 ** 28 ** PWM	OC0           
	TIMER1A 	, // PB 5 ** 29 ** PWM	OC1A          
	TIMER1B 	, // PB 6 ** 30 ** PWM	OC1B          
	TIMER1C 	, // PB 7 ** 31 ** PWM	OC2/OC1C      
	NOT_ON_TIMER	, // PE 0 ** 32 ** USART0_RX          
	NOT_ON_TIMER	, // PE 1 ** 33 ** USART0_TX          
	NOT_ON_TIMER	, // PE 2 ** 34 ** D34	              
	TIMER3A		, // PE 3 ** 35 ** PWM OC3A           
	TIMER3B 	, // PE 4 ** 36 ** PWM OC3B INT4      
	TIMER3C 	, // PE 5 ** 37 ** PWM OC3C INT5      
	NOT_ON_TIMER	, // PE 6 ** 38 ** INT6               
	NOT_ON_TIMER	, // PE 7 ** 39 ** INT7               
	NOT_ON_TIMER	, // PF 0 ** 40 ** D40 A0             
	NOT_ON_TIMER	, // PF 1 ** 41 ** D41 A1             
	NOT_ON_TIMER	, // PF 2 ** 42 ** D42 A2             
	NOT_ON_TIMER	, // PF 3 ** 43 ** D43 A3             
	NOT_ON_TIMER	, // PF 4 ** 44 ** D44 A4             
	NOT_ON_TIMER	, // PF 5 ** 45 ** D45 A5             
	NOT_ON_TIMER	, // PF 6 ** 46 ** D46 A6             
	NOT_ON_TIMER	, // PF 7 ** 47 ** D47 A7             
	NOT_ON_TIMER	, // PG 0 ** 48 ** D48	              
	NOT_ON_TIMER	, // PG 1 ** 49 ** D49	              
	NOT_ON_TIMER	, // PG 2 ** 50 ** D50	              
};
#else
// these arrays map port names (e.g. port B) to the
// appropriate addresses for various functions (e.g. reading
// and writing)
const uint16_t PROGMEM port_to_mode_PGM[] = {
	NOT_A_PORT,
	NOT_A_PORT,
	&DDRB,
	&DDRC,
	&DDRD,
};

const uint16_t PROGMEM port_to_output_PGM[] = {
	NOT_A_PORT,
	NOT_A_PORT,
	&PORTB,
	&PORTC,
	&PORTD,
};

const uint16_t PROGMEM port_to_input_PGM[] = {
	NOT_A_PORT,
	NOT_A_PORT,
	&PINB,
	&PINC,
	&PIND,
};

const uint8_t PROGMEM digital_pin_to_port_PGM[] = {
	PD, /* 0 */
	PD,
	PD,
	PD,
	PD,
	PD,
	PD,
	PD,
	PB, /* 8 */
	PB,
	PB,
	PB,
	PB,
	PB,
	PC, /* 14 */
	PC,
	PC,
	PC,
	PC,
	PC,
};

const uint8_t PROGMEM digital_pin_to_bit_mask_PGM[] = {
	_BV(0), /* 0, port D */
	_BV(1),
	_BV(2),
	_BV(3),
	_BV(4),
	_BV(5),
	_BV(6),
	_BV(7),
	_BV(0), /* 8, port B */
	_BV(1),
	_BV(2),
	_BV(3),
	_BV(4),
	_BV(5),
	_BV(0), /* 14, port C */
	_BV(1),
	_BV(2),
	_BV(3),
	_BV(4),
	_BV(5),
};

const uint8_t PROGMEM digital_pin_to_timer_PGM[] = {
	NOT_ON_TIMER, /* 0 - port D */
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	// on the ATmega168, digital pin 3 has hardware pwm
#if defined(__AVR_ATmega8__)
	NOT_ON_TIMER,
#else
	TIMER2B,
#endif
	NOT_ON_TIMER,
	// on the ATmega168, digital pins 5 and 6 have hardware pwm
#if defined(__AVR_ATmega8__)
	NOT_ON_TIMER,
	NOT_ON_TIMER,
#else
	TIMER0B,
	TIMER0A,
#endif
	NOT_ON_TIMER,
	NOT_ON_TIMER, /* 8 - port B */
	TIMER1A,
	TIMER1B,
#if defined(__AVR_ATmega8__)
	TIMER2,
#else
	TIMER2A,
#endif
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER, /* 14 - port C */
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
	NOT_ON_TIMER,
};
#endif
